#include "maketree.h"

/************************************************************************/
/* NOTE: Compiled with Turbo C++ professional. Your mileage may vary.   */
/************************************************************************/

/************************************************************************/
/* MAKETREE - Copyright (HA!) 1991 by Richard Lawrence                  */
/*            If you like this utility, call the Hallucination at       */
/*                      703-425-5824                                    */
/*            2 Gigabytes of storage, the largest message area in the   */
/*            world!                                                    */
/*                                                                      */
/*            A utility to create a directory tree based on the output  */
/*            a previously executed DOS TREE command. It is assumed you */
/*            have executed the TREE command and redirected the output  */
/*            to a file, ie, TREE > INFILE. MAKETREE will take INFILE as*/
/*            a parameter and recreate the entire directory structure of*/
/*            that TREE file.                                           */
/************************************************************************/

/************************************************************************/
/* Global variable list:                                                */
/* Variable                 Purpose                                     */
/* -----------------------  ------------------------------------------  */
/* INFILE_handle            MS-DOS file handle for the input file       */
/* length                   length of the input file returned by DOS    */
/* membuffer                Pointer to the buffer area holding INFILE   */
/* counter                  Reference pointer within MEMBUFFER area     */
/************************************************************************/

int             INFILE_handle=0,length=0;
unsigned char   *membuffer=NULL,*counter=NULL;

/************************************************************************/
/* void init_variables(int, char*)                                      */
/* This function opens the file passed via argv[1] from the command line*/
/* and reads it into buffer of LENGTH bytes pointed to by MEMBUFFER. It */
/* then sets COUNTER equal to the first significant character of the    */
/* buffer area (that is, the first character of the first line of the   */
/* actual tree).                                                        */
/************************************************************************/

void init_variables(int argc,char *filename)
{
    if( (argc==1) | (argc>2) )
    {
        printf(WHATIS);
        terminate_program(USAGETEXT,5);
    }
    if( (INFILE_handle=_open(filename,O_RDONLY))<0 )
        terminate_program("Could not open specified INFILE. Check path and spelling.\n",15);
    length=filelength(INFILE_handle);
    if( (membuffer=malloc(length))==NULL )
        terminate_program("Could not alloc memory for membuffer.\n",20);
    if( _read(INFILE_handle,membuffer,length)!=length )
        terminate_program("Problem reading from specified INFILE - incomplete read.\n",15);
    _close(INFILE_handle);
    counter=membuffer;
    do counter++; while( (*counter<128) && (counter<membuffer+length) );
    if(counter==membuffer+length)
        terminate_program("Error in INFILE - not a DOS 4.XX or 5.X TREE output?\n",10);
    else
        counter--;
}

/************************************************************************/
/* int main(int, char**)                                                */
/* Where all the real work is done. Basically, COUNTER points towards   */
/* the current character in the buffer, FULLPATH holds the entire       */
/* pathname, and HOLD holds the current directory being added. HOWDEEP  */
/* tells how many levels deep in directories we are (the root dir is 0, */
/* subdirectories directly off of root are 1, so on). CURDEEP will show */
/* how deep the new directory being read from the tree is.              */
/************************************************************************/

int main(int argc,char *argv[])
{
    unsigned char   fullpath[254]="\\",hold[13]="\0",temp,curdeep,howdeep=0;

    init_variables(argc,argv[1]);
    while( counter<(membuffer+length-1) )
    {
        counter++;
        if( (*counter<127) && (*counter!=32) )
        {
            printf("Unexpected text encountered in INFILE - parse error. Text is:\n");
            _write(1,counter,30);
            printf("\nFirst byte of this text has an ASCII value of %i\nA value greater than 128 or a space was expected.\n",*counter);
            terminate_program("\nINFILE is corrupted or is from non-supported DOS version.\n",10);
        }
        curdeep=0;
        while( (*counter>127) || (*counter==32) )
        {
            counter+=4;
            curdeep++;
        }
        if( *counter<33 || *counter>127 )
            terminate_program("\nUnexpected text discovered in INFILE.\n",10);
        for(temp=0; (temp<13) && (counter[temp]!=13) ;temp++)
            hold[temp]=counter[temp];
        if(temp==13)
            terminate_program("\nERROR - maximum filename length (11 chars) exceeded.\n",20);
        hold[temp]=0;
        if( curdeep>howdeep )
            howdeep=curdeep;
        else
            if( (howdeep>curdeep) || (howdeep==curdeep) )
            {
                howdeep=0;
                for(temp=0;howdeep!=curdeep;temp++)
                    if( fullpath[temp]==92 )
                        howdeep++;
                if(temp==0) temp=1;
                fullpath[temp]=0;
            }
        strcat(fullpath,hold);
        printf("\nCreating %s",fullpath);
        if( mkdir(fullpath) )
            if( chdir(fullpath) )
                terminate_program(" ERROR - Problem creating directory.\n",15);
        strcat(fullpath,"\\");
        do counter++; while(*counter!=10);
    }
    terminate_program("\nProgram completed normally.\n",0);
    return(0);
}

/************************************************************************/
/* terminate_program(char *,int):                                       */
/*                                                                      */
/* Frees any allocated bufferspace, and closes any open files. Prints   */
/* message to user (may or may not be an error).                        */
/************************************************************************/


void terminate_program(char *error_string,int error_value)
{
    if( membuffer )
        free(membuffer);
    membuffer=NULL;
    if( INFILE_handle )
        _close( INFILE_handle );
    printf(error_string);
    _exit(error_value);
}
